<?php

namespace HS\IncomingMail\Processors;

use Nekman\LuhnAlgorithm\Contract\LuhnAlgorithmExceptionInterface;
use Nekman\LuhnAlgorithm\LuhnAlgorithmFactory;
use Nekman\LuhnAlgorithm\Number;

class Redactor
{
    /**
     * // Attempt to match social security numbers
     * @var string
     */
    public string $socialRegex = '/\b\d{3}[- ]?\d{2}[- ]?\d{4}\b/';

    /**
     * // match 13-16 digit numbers that do not start with a + to remove phone numbers
     * @var string
     */
    public string $ccRegex = '/(?<!\+)\b(?:\d[ -]*){13,16}\b/';

    public function redact($string)
    {
        $redactions = $this->findRedactions($string);
        if ($redactions) {
            return $this->doRedaction($string, $redactions);
        }
        return $string;
    }

    public function luhnVerify($number)
    {
        try {
            $validator = Number::fromString($number);
            return LuhnAlgorithmFactory::create()->isValid($validator);
        } catch (\Exception|LuhnAlgorithmExceptionInterface $e) {
            return false;
        }
    }

    public function doRedaction($string, $redactions)
    {
        $redactions->each(function($redaction) use (&$string){
            if ($this->luhnVerify($redaction)) {
                $string = $this->replaceRedactions($redaction, $string);
            } else if (preg_match($this->socialRegex, $redaction)){
                $string = $this->replaceRedactions($redaction, $string);
            }
        });

        return $string;
    }

    protected function replaceRedactions($redaction, $string): string
    {
        return str_replace($redaction, str_repeat('█', mb_strlen($redaction)), $string);
    }

    public function creditCardRedactionOn(): bool
    {
        return (hs_setting('cHD_REDACTOR_CC') == 1);
    }

    public function socialRedactionOn(): bool
    {
        return (hs_setting('cHD_REDACTOR_SS') == 1);
    }

    public function findRedactions($string)
    {
        if ($this->creditCardRedactionOn() && $this->socialRedactionOn()) {
            $out = array_merge($this->matchCreditCards($string), $this->matchSSNs($string));
        } elseif ($this->creditCardRedactionOn()) {
            $out = $this->matchCreditCards($string);
        } elseif ($this->socialRedactionOn()) {
            $out = $this->matchSSNs($string);
        } else {
            return false;
        }

        $items = collect($out)->collapse()->transform(function($item, $value){
            return trim($item);
        });
        return ($items->count() > 0) ? $items : false;
    }

    protected function matchCreditCards($string): array
    {
        preg_match_all($this->ccRegex, $string, $out);
        return $out;
    }

    protected function matchSSNs($string): array
    {
        preg_match_all($this->socialRegex, $string, $out);
        return $out;
    }
}
